/******************************************************************************
 * The MIT License (MIT)
 *
 * Copyright (c) 2019-2023 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 ******************************************************************************/

#pragma once

#ifndef NOMINMAX
#define NOMINMAX
#endif

#define VK_NO_PROTOTYPES

#if defined(__LP64__) || defined(_WIN64) || (defined(__x86_64__) && !defined(__ILP32__)) || \
    defined(_M_X64) || defined(__ia64) || defined(_M_IA64) || defined(__aarch64__) ||       \
    defined(__powerpc64__) || (defined(__riscv) && __riscv_xlen == 64)

#else

// make handles typed even on 32-bit, by relying on C++
#define VK_DEFINE_NON_DISPATCHABLE_HANDLE(obj) \
  struct obj                                   \
  {                                            \
    obj() : handle(0)                          \
    {                                          \
    }                                          \
    obj(uint64_t x) : handle(x)                \
    {                                          \
    }                                          \
    bool operator==(const obj &other) const    \
    {                                          \
      return handle == other.handle;           \
    }                                          \
    bool operator<(const obj &other) const     \
    {                                          \
      return handle < other.handle;            \
    }                                          \
    bool operator!=(const obj &other) const    \
    {                                          \
      return handle != other.handle;           \
    }                                          \
    bool operator==(const int &other) const    \
    {                                          \
      return handle == other;                  \
    }                                          \
    bool operator!=(const int &other) const    \
    {                                          \
      return handle != other;                  \
    }                                          \
    operator bool() const                      \
    {                                          \
      return handle != 0;                      \
    }                                          \
    operator uint64_t() const                  \
    {                                          \
      return handle;                           \
    }                                          \
    uint64_t handle;                           \
  };

#endif

// first include our local vulkan to ensure it's used over any system header
#include "official/vulkan/vulkan.h"

// then include volk
#include "3rdparty/volk/volk.h"

// finally VMA
#include "3rdparty/VulkanMemoryAllocator/vk_mem_alloc.h"
